'
'  LIFE EDITOR
'  Easily create or edit patterns to be used in LIFE.
'
'   by David Karlov
'  dkarlov@acs.itd.uts.edu.au
'  dkarlov@rocketmail.com
'
'  Use mouse or keyboard to edit your life pattern:
'
'  MOUSE             KEYBOARD       EFFECT
'  Left Button       Space Bar      Create colour 1 life cell
'  Right Button      Enter          Create colour 2 life cell
'  Both Buttons      Delete         Remove life cell
'
'  OTHER KEYS:
'  G       change grid size
'  S       save your pattern
'  L       load a pattern to edit
'  N       new pattern (lose all data)
'  Q       quit LIFEEDIT
'
' NOTE -- start QB with command line option /L QB to load library, otherwise
'         this program will not run in QuickBASIC.....


TYPE RegType
	  ax    AS INTEGER
	  bx    AS INTEGER
	  cx    AS INTEGER
	  dx    AS INTEGER
	  bp    AS INTEGER
	  si    AS INTEGER
	  di    AS INTEGER
	  flags AS INTEGER
END TYPE

DEFINT A-Z
DECLARE SUB LoadLif ()
DECLARE SUB SaveLif ()
DECLARE SUB NewLif ()
DECLARE SUB DrawGrid ()
DECLARE SUB EditGrid ()
DECLARE SUB GetXMYM ()
DECLARE SUB DrawBox (x1 AS INTEGER, y1 AS INTEGER, c AS INTEGER)
DECLARE SUB FillBox (x1 AS INTEGER, y1 AS INTEGER, c AS INTEGER)
DECLARE FUNCTION GetInput$ ()
DECLARE FUNCTION Sure ()

DECLARE SUB MouseDriver (ax AS INTEGER, bx AS INTEGER, cx AS INTEGER, dx AS INTEGER)
DECLARE SUB MouseShow ()
DECLARE SUB MouseHide ()
DECLARE SUB MouseStatus (lb AS INTEGER, rb AS INTEGER, mx AS INTEGER, my AS INTEGER)
DECLARE SUB MousePut (X AS INTEGER, Y AS INTEGER)
DECLARE FUNCTION MouseReset ()

DIM SHARED Reg AS RegType
DIM SHARED MouseExists AS INTEGER
DIM SHARED Changed AS INTEGER
DIM SHARED JumpLoad AS INTEGER


DIM SHARED Grid(0 TO 59, 0 TO 35) AS INTEGER

DIM SHARED GridSize AS STRING
DIM SHARED XM AS INTEGER, YM AS INTEGER
DIM SHARED Col(0 TO 2) AS INTEGER
Col(0) = 0: Col(1) = 14: Col(2) = 9
GridSize = "Small"
GetXMYM

SCREEN 13: CLS : COLOR 7

MouseExists = MouseReset
IF MouseExists THEN MouseHide: MousePut 320, 100


IF COMMAND$ <> "" THEN
	JumpLoad = 1
	LoadLif
ELSE
	DrawGrid
END IF

EditGrid

SUB DrawBox (x1 AS INTEGER, y1 AS INTEGER, c AS INTEGER)

	IF x1 >= 0 AND x1 < XM AND y1 >= 0 AND y1 < YM THEN
		IF GridSize = "Small" THEN
			LINE (x1 * 20 + 10, y1 * 20)-(x1 * 20 + 30, y1 * 20 + 20), c, B
		ELSEIF GridSize = "Big" THEN
			LINE (x1 * 10 + 10, y1 * 10)-(x1 * 10 + 20, y1 * 10 + 10), c, B
		ELSE
			LINE (x1 * 5 + 10, y1 * 5)-(x1 * 5 + 15, y1 * 5 + 5), c, B
		END IF
	END IF
	
END SUB

SUB DrawGrid

	LOCATE 25, 1: PRINT SPACE$(40);
	LOCATE 25, 1: PRINT "(L)oad (S)ave (N)ew (Q)uit (G)rid: "; GridSize;

	LINE (10, 0)-(310, 180), 0, BF
	
	IF GridSize = "Small" THEN s1 = 20 ELSE IF GridSize = "Big" THEN s1 = 10 ELSE s1 = 5
	FOR y1 = 0 TO 180 STEP s1
		LINE (10, y1)-(310, y1), 8
	NEXT y1
	FOR x1 = 10 TO 310 STEP s1
		LINE (x1, 0)-(x1, 180), 8
	NEXT x1

	FOR x1 = 0 TO XM - 1
		FOR y1 = 0 TO YM - 1
			FillBox x1, y1, Grid(x1, y1)
		NEXT y1
	NEXT x1

END SUB

SUB EditGrid

Changed = 0
X = -1: Y = -1
nx = 0: ny = 0

DO
	i$ = UCASE$(INKEY$)
	IF MouseExists THEN
		MouseStatus lb, rb, mx, my
		IF mx < 320 - 600 \ XM THEN
			MousePut mx + 1200 \ XM, my
			IF nx > 0 THEN nx = X - 1
		ELSEIF mx > 320 + 600 \ XM THEN
			MousePut mx - 1200 \ XM, my
			IF nx < XM THEN nx = X + 1
		END IF
		IF my < 100 - 180 \ YM THEN
			MousePut mx, my + 360 \ YM
			ny = (Y - 1 + YM) MOD YM
		ELSEIF my > 100 + 180 \ YM THEN
			MousePut mx, my - 360 \ YM
			ny = (Y + 1) MOD YM
		END IF
		IF lb OR rb THEN i$ = " "
	END IF
  
	
	SELECT CASE i$
		CASE CHR$(0) + "G", "7"
			nx = (X - 1 + XM) MOD XM
			ny = (Y - 1 + YM) MOD YM
		CASE CHR$(0) + "H", "8"
			ny = (Y - 1 + YM) MOD YM
		CASE CHR$(0) + "I", "9"
			nx = (X + 1) MOD XM
			ny = (Y - 1 + YM) MOD YM
		CASE CHR$(0) + "K", "4"
			nx = (X - 1 + XM) MOD XM
		CASE CHR$(0) + "M", "6"
			nx = (X + 1) MOD XM
		CASE CHR$(0) + "O", "1"
			nx = (X - 1 + XM) MOD XM
			ny = (Y + 1) MOD YM
		CASE CHR$(0) + "P", "2"
			ny = (Y + 1) MOD YM
		CASE CHR$(0) + "Q", "3"
			nx = (X + 1) MOD XM
			ny = (Y + 1) MOD YM
		CASE " ", CHR$(13), "1", "2"
			Changed = 1
			IF MouseExists THEN
				IF lb AND rb THEN
					Grid(X, Y) = 0
				ELSEIF lb THEN
					Grid(X, Y) = 1
				ELSEIF rb THEN
					Grid(X, Y) = 2
				END IF
			ELSE
				IF i$ = " " OR i$ = "1" THEN
					IF Grid(X, Y) = 1 THEN Grid(X, Y) = 0 ELSE Grid(X, Y) = 1
				ELSE
					IF Grid(X, Y) = 2 THEN Grid(X, Y) = 0 ELSE Grid(X, Y) = 2
				END IF
			END IF
			FillBox X, Y, Grid(X, Y)
		CASE "0", CHR$(0) + "S"
			Grid(X, Y) = 0
			FillBox X, Y, 0
		CASE "N"
			X = -1: Y = -1: NewLif
		CASE "S"
			SaveLif
		CASE "L"
			X = -1: Y = -1: LoadLif
		CASE "G"
			X = -1: Y = -1
			IF GridSize = "Small" THEN
				GridSize = "Big"
			ELSEIF GridSize = "Big" THEN
				GridSize = "Huge"
			ELSE
				GridSize = "Small"
			END IF
			GetXMYM
			DrawGrid
		CASE "Q", CHR$(27)
			LOCATE 24, 1: PRINT "QUIT: ";
			IF Sure = 0 THEN
				LOCATE 24, 1: PRINT "Quit cancelled            ";
			ELSE
				SCREEN 0, 0, 0: WIDTH 80: CLS : END
			END IF
	END SELECT
	IF nx <> X OR ny <> Y THEN
		DrawBox X, Y, 8
		DrawBox nx, ny, 15
		X = nx: Y = ny
	END IF

LOOP


END SUB

SUB FillBox (x1 AS INTEGER, y1 AS INTEGER, c AS INTEGER)
' c = 0, 1, or 2

	c2 = Col(c)

	IF x1 >= 0 AND x1 < XM AND y1 >= 0 AND y1 < YM THEN
		IF GridSize = "Small" THEN
			LINE (x1 * 20 + 11, y1 * 20 + 1)-(x1 * 20 + 29, y1 * 20 + 19), Col(c), BF
		ELSEIF GridSize = "Big" THEN
			LINE (x1 * 10 + 11, y1 * 10 + 1)-(x1 * 10 + 19, y1 * 10 + 9), Col(c), BF
		ELSE
			LINE (x1 * 5 + 11, y1 * 5 + 1)-(x1 * 5 + 14, y1 * 5 + 4), Col(c), BF
		END IF
	END IF

END SUB

FUNCTION GetInput$

a$ = ""
y1 = CSRLIN: x1 = POS(0)


DO
	i$ = INKEY$
	SELECT CASE i$
		CASE CHR$(33) TO CHR$(127)
			IF LEN(a$) < 8 THEN a$ = a$ + i$
			LOCATE y1, x1: PRINT a$;
		CASE CHR$(8)
			IF LEN(a$) > 0 THEN a$ = LEFT$(a$, LEN(a$) - 1)
			LOCATE y1, x1: PRINT a$; " ";
		CASE CHR$(13)
			IF LEN(a$) > 0 THEN EXIT DO
		CASE CHR$(27)
			a$ = "": EXIT DO
	END SELECT
LOOP


GetInput$ = a$

END FUNCTION

SUB GetXMYM

IF GridSize = "Small" THEN
	XM = 15: YM = 9
ELSEIF GridSize = "Big" THEN
	XM = 30: YM = 18
ELSEIF GridSize = "Huge" THEN
	XM = 60: YM = 36
ELSE
	PRINT "Program error -- Invalid grid size"
	STOP
END IF

END SUB

SUB LoadLif

IF JumpLoad = 0 THEN

	LOCATE 24, 1: PRINT "LOAD: ";
	IF Sure = 0 THEN LOCATE 24, 1: PRINT "Load cancelled            "; : EXIT SUB

	i$ = ""
	DO
		LOCATE 24, 1: PRINT "Filename to load:         .LIF";
		COLOR 15: LOCATE 24, 19: F$ = UCASE$(GetInput$): COLOR 7
		LOCATE 24, 1: PRINT SPACE$(39);
		IF F$ = "" THEN LOCATE 24, 1: PRINT "Load cancelled"; : EXIT SUB
		Exists = 0
		OPEN F$ + ".LIF" FOR RANDOM AS #1 LEN = 10
		FIELD #1, 10 AS a$: GET #1, 1
		IF a$ <> STRING$(10, 0) THEN Exists = 1
		CLOSE #1
		IF Exists = 0 THEN
			LOCATE 24, 1: PRINT F$; ".LIF not found, press any key";
			DO: LOOP WHILE INKEY$ = ""
			LOCATE 24, 1: PRINT SPACE$(39);
		END IF
	LOOP WHILE Exists = 0
ELSE
	JumpLoad = 0
	F$ = COMMAND$
	IF INSTR(F$, ".") THEN F$ = LEFT$(LEFT$(F$, INSTR(F$, ".") - 1), 8)
	IF LEN(F$) = 0 THEN DrawGrid: EXIT SUB
	OPEN F$ + ".LIF" FOR RANDOM AS #1 LEN = 10
	FIELD #1, 10 AS a$: GET #1, 1
	IF a$ = STRING$(10, 0) THEN CLOSE : DrawGrid: EXIT SUB
	CLOSE #1
END IF

DIM Temp(1 TO 3, 2200) AS INTEGER
c = 0: xl = 30: yl = 19: xb = -1: yb = -1
OPEN F$ + ".LIF" FOR BINARY AS #1
DO
	c = c + 1
	GET 1, , x1
	IF NOT EOF(1) THEN GET 1, , y1 ELSE EXIT DO
	IF NOT EOF(1) THEN GET 1, , c1 ELSE EXIT DO
	Temp(1, c) = x1: Temp(2, c) = y1: Temp(3, c) = c1
	IF x1 < xl THEN xl = x1
	IF x1 > xb THEN xb = x1
	IF y1 < yl THEN yl = y1
	IF y1 > yb THEN yb = y1
LOOP UNTIL EOF(1)
CLOSE #1
xa = CINT((xl + xb) / 2)
ya = CINT((yl + yb) / 2)

IF xb - xl >= 30 OR yb - yl >= 18 THEN
	GridSize = "Huge"
ELSEIF xb - xl >= 15 OR yb - yl >= 9 THEN
	GridSize = "Big"
ELSE
	GridSize = "Small"
END IF
GetXMYM

FOR x1 = 0 TO 59
	FOR y1 = 0 TO 35
		Grid(x1, y1) = 0
	NEXT y1
NEXT x1

Lost = 0
FOR t = 1 TO c - 1
	x1 = Temp(1, t) - xa + CINT((XM - 1) / 2)
	y1 = Temp(2, t) - ya + CINT((YM - 1) / 2)
	IF x1 >= 0 AND y1 >= 0 AND x1 < XM AND y1 < YM THEN
		Grid(x1, y1) = Temp(3, t)
	ELSE
		Lost = 1
	END IF
NEXT t
IF Lost = 1 THEN
	LOCATE 24, 1: PRINT "Data too big for grid, some data lost.";
	WHILE INKEY$ = "": WEND
	LOCATE 24, 1: PRINT SPACE$(39);
END IF
LOCATE 24, 1: PRINT F$; ".LIF loaded.";

DrawGrid

END SUB

SUB MouseDriver (ax AS INTEGER, bx AS INTEGER, cx AS INTEGER, dx AS INTEGER)
' Run the machine language program with various pararmeters
  
	Reg.ax = ax: Reg.bx = bx: Reg.cx = cx: Reg.dx = dx: Reg.bp = 0: Reg.si = 0: Reg.di = 0
	CALL INTERRUPT(&H33, Reg, Reg)
	ax = Reg.ax: bx = Reg.bx: cx = Reg.cx: dx = Reg.dx

END SUB

SUB MouseHide
' BIOS interrupt 33H, function 02
' Make the mouse cursor vanish

	MouseDriver 2, 0, 0, 0

END SUB

SUB MousePut (X AS INTEGER, Y AS INTEGER)
' BIOS interrupt 33H, function 04
' Move the mouse cursor to a new location

	MouseDriver 4, 0, X, Y

END SUB

FUNCTION MouseReset
' BIOS interrupt 33H, function 00
' Reset the mouse, and determine if it exists
  
	ax = 0
	MouseDriver ax, 0, 0, 0
	MouseReset = ax

END FUNCTION

SUB MouseShow
' BIOS interrupt 33H, function 01
' Enable the mouse cursor to be seen

	MouseDriver 1, 0, 0, 0

END SUB

SUB MouseStatus (lb AS INTEGER, rb AS INTEGER, mx AS INTEGER, my AS INTEGER)
' BIOS interrupt 33H, function 03
' Get the current status of the mouse; ie buttons pressed and its position

	MouseDriver 3, bx, cx, dx
	lb = ((bx AND 1) > 0)
	rb = ((bx AND 2) > 0)
	mx = cx
	my = dx

END SUB

SUB NewLif

	LOCATE 24, 1: PRINT "NEW: ";
	IF Sure = 0 THEN LOCATE 24, 1: PRINT "New cancelled            "; : EXIT SUB

	FOR x1 = 0 TO 59
		FOR y1 = 0 TO 35
			Grid(x1, y1) = 0
		NEXT y1
	NEXT x1

	DrawGrid
	Changed = 0

END SUB

SUB SaveLif

i$ = ""
DO
	LOCATE 24, 1: PRINT "Filename to save under:         .LIF";
	COLOR 15: LOCATE 24, 25: F$ = UCASE$(GetInput$): COLOR 7
	LOCATE 24, 1: PRINT SPACE$(39);
	IF F$ = "" THEN LOCATE 24, 1: PRINT "Save cancelled"; : EXIT SUB
	Exists = 0: i$ = ""
	OPEN F$ + ".LIF" FOR RANDOM AS #1 LEN = 10
	FIELD #1, 10 AS a$: GET #1, 1
	IF a$ <> STRING$(10, 0) THEN Exists = 1
	CLOSE #1
	IF Exists = 1 THEN
		LOCATE 24, 1: PRINT F$; ".LIF exists, overwrite? (Y/N)";
		DO
			i$ = UCASE$(INKEY$)
		LOOP UNTIL i$ = "Y" OR i$ = "N"
		LOCATE 24, 1: PRINT SPACE$(39);
	END IF
LOOP WHILE i$ = "N"


OPEN F$ + ".LIF" FOR OUTPUT AS #1: CLOSE #1  ' delete all info in file
OPEN F$ + ".LIF" FOR BINARY AS #1

FOR X = 0 TO XM - 1
	FOR Y = 0 TO YM - 1
		IF Grid(X, Y) THEN
			PUT 1, , X
			PUT 1, , Y
			PUT 1, , Grid(X, Y)
		END IF
	NEXT Y
NEXT X

CLOSE #1
LOCATE 24, 1: PRINT F$; ".LIF saved";
Changed = 0

END SUB

FUNCTION Sure

IF Changed = 0 THEN
	Sure = 1
	LOCATE 24, 1: PRINT SPACE$(39);
	EXIT FUNCTION
END IF

PRINT "Are you sure? (Y/N)";
DO
	i$ = UCASE$(INKEY$)
LOOP UNTIL i$ = "Y" OR i$ = "N"

IF i$ = "Y" THEN Sure = 1 ELSE Sure = 0

END FUNCTION

